#ifndef SRC_CORE_DATA_BEHAVIOUR_HPP_
#define SRC_CORE_DATA_BEHAVIOUR_HPP_

#include <vector>
#include <variant>
#include "src/core/enums.hpp"
#include "src/core/vector.hpp"

struct SimpleBehaviourMovement {
	SimpleBehaviourInstruction type;
	short frames;
	Vec3f speed;
	Angle3s angularVelocity;

	static const SimpleBehaviourMovement &Default();
};

struct SimpleBehaviour {
	uint segmentedAddress;

	CollisionDistanceMode collisionDistanceMode;
	short renderDistance; // -1 means infinite
	short collisionDistance;

	bool movesMario;
	bool resetAfterLoop;
	std::vector<SimpleBehaviourMovement> movementLoop;

	static const SimpleBehaviour &Default();

	inline short getComputedCollisionDistance( ushort minRadius ) const {
		switch( collisionDistanceMode ) {
			case CollisionDistanceMode::Minimal:
				return (minRadius <= 0x7FFFu - 160u) ? ((short)minRadius + 160) : -1;
			case CollisionDistanceMode::Suggested:
				return (minRadius <= 0x7FFFu - 1000u) ? ((short)minRadius + 1000) : -1;
			case CollisionDistanceMode::Infinite:
				return -1;
			case CollisionDistanceMode::Manual:
				return collisionDistance;
			default:
				return 0;
		}
	}
};

struct Behaviour {
	BehaviourType type;
	uint romAddress; // for BehaviourType::Vanilla
	SimpleBehaviour autoGen; // for BehaviourType::Autogenerated
};

namespace JsonSerializer {
	template<> void serialize<SimpleBehaviourMovement>( JsonWriter &jw, const SimpleBehaviourMovement &obj );
	template<> SimpleBehaviourMovement parse<SimpleBehaviourMovement>( const Json &json );

	template<> void serialize<Behaviour>( JsonWriter &jw, const Behaviour &obj );
	template<> Behaviour parse<Behaviour>( const Json &json );
}

#endif /* SRC_CORE_DATA_BEHAVIOUR_HPP_ */
